------------------------------------------------------------------------
-- ADC SPI interface. Not very special, just 10-bit SPI interface,    --
-- both receive/transmit register are here. Kinda automatic by doing  --
-- the entire sequence all by itself after command is written to THR. --
-- Does not get idle immediately after data transfer is complete,     --
-- only after a programmable delay (generic -> convtime_val) required --
-- for the ADC to complete conversion.                                --
--                                                                    --
-- Made for a very specific chip, TI TLV1544 that is installed on BRK --
-- board and hooked up to LSS21 on SPI2 interface.                    --
------------------------------------------------------------------------

--------------------------------------------------------------------------
--			LICENSE and Terms of Use.			--
--									--
-- First of all, this is NOT GPL or Free Software. It is open so you	--
-- have the VHDL source code. There is also the source code for the	--
-- Windows Engine for Mach3 CNC software available from KSI Labs, LLC	--
-- website, http://www.ksilabs.com.					--
--									--
-- You are free to use this particular VHDL code, make modifications to	--
-- customize it for your own needs, fix inevitable bugs, port it to 	--
-- other CNC software and do anything else as long as your derivative	--
-- work that is based on this code or uses parts or pieces thereof is	--
-- made to be used with any KSI Labs, LLC products.			--
--									--
-- We do NOT grant you a permission to use any parts of this code in	--
-- any products that are designed for products not manufactured or	--
-- distributed by KSI Labs, LLC. We might give such a permission on	--
-- case-by-case basis but it is not legal to use this code or bits and	--
-- pieces thereof with products not coming from KSI Labs, LLC without	--
-- our explicit written permission.					--
--									--
-- The original copyright notices must be included in all derivative	--
-- works. You do NOT have to ask our permission to make any changes to	--
-- this code and you are not under obligation to give us any of your	--
-- work back as long as you are using it for your own needs.		--
--									--
-- However we do NOT give our permission to distribute your derivative	--
-- work unless all original copyrights are retained, all the conditions	--
-- of this license are met (use only with KSI Labs, LLC products) and	--
-- full source code for such derivative work is available for free to	--
-- everybody without any further conditions including but not limited	--
-- to signing any kinds of NDA. We explicitely deny any permission to	--
-- distribute any derivative work based on this code with closed source	--
-- binary-only fragments or include bits or pieces of this code in such	--
-- products. That essentially means that it is NOT legal to use it with --
-- ANY proprietary HDL megafunctions, megacores, etc. if the resulting	--
-- product is distributed. It IS legal to use whatever you want if the	--
-- resulting product is made for use with a KSI Labs, LLC product AND	--
-- that resulting product is not distributed. Exclusions can be granted --
-- by KSI Labs, LLC on case-by-case basis.				--
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity tlv1544_adc is
generic (
	clkdiv_val	: unsigned(8 downto 0) := b"0_0010_0010";
	convtime_val	: unsigned(8 downto 0) := b"1_1111_0100"
);
port (
	adc_idle	: out std_logic;
	adc_rhr		: out std_logic_vector(9 downto 0);
	adc_thr		: in std_logic_vector(9 downto 0);

	ss_pin		: out std_logic;
	sclk_pin	: out std_logic;
	mosi_pin	: out std_logic;
	miso_pin	: in std_logic;
	adc_sel		: in std_logic;
	wr		: in std_logic;
	clk		: in std_logic;
	rst		: in std_logic
);
end tlv1544_adc;

architecture guts of tlv1544_adc is

	-------------
	-- Signals --
	-------------

	type fsm_state is (IDLE, START, LATCH, SHIFT, CONVERSION);
	signal fsm_cur	: fsm_state;
	signal fsm_nxt	: fsm_state;

	signal thr		: std_logic_vector(9 downto 0);
	signal rhr		: std_logic_vector(9 downto 0);
	signal clkdiv		: unsigned(8 downto 0);
	signal bit_cntr		: unsigned(3 downto 0);
	signal do_shift		: std_logic;
	signal do_rd_bit	: std_logic;
	signal adc_write	: std_logic;

begin

	--------------------------------------
	-- Top half of the FSM (sequential) --
	--------------------------------------
	fsm_top : process(rst, clk)
	begin
		if (rst='1') then
			fsm_cur <= IDLE;
		elsif (rising_edge(clk)) then
			if (fsm_cur=IDLE) then
				if (adc_write='1') then
					thr <= adc_thr;
					fsm_cur <= START;
					bit_cntr <= (others => '0');
					clkdiv <= clkdiv_val;
				end if;
			elsif (clkdiv=(clkdiv'range => '0')) then
				if (do_rd_bit='1') then
					rhr <= rhr(8 downto 0) & miso_pin;
				end if;
				if (do_shift='1') then
					thr <= thr(8 downto 0) & '0';
					bit_cntr <= bit_cntr + 1;
				end if;
				if (fsm_nxt=CONVERSION) then
					clkdiv <= convtime_val;
				else
					clkdiv <= clkdiv_val;
				end if;
				fsm_cur <= fsm_nxt;
			else
				clkdiv <= clkdiv - 1;
			end if;
		end if;
	end process;


	--------------------------------------------
	-- Bottom half of the FSM (combinatorial) --
	--------------------------------------------
	fsm_st_bh : process(fsm_cur, bit_cntr)
	begin
		case fsm_cur is
			when IDLE =>
				fsm_nxt <= IDLE;
			when START =>
				fsm_nxt <= LATCH;
			when LATCH =>
				if (bit_cntr=x"9") then
					fsm_nxt <= CONVERSION;
				else
					fsm_nxt <= SHIFT;
				end if;
			when SHIFT =>
				fsm_nxt <= LATCH;
			when CONVERSION =>
				fsm_nxt <= IDLE;
		end case;
	end process;


	adc_write <= adc_sel and wr;

	adc_idle <= '1' when (fsm_cur=IDLE) else '0';
	do_shift <= '1' when (fsm_nxt=SHIFT) else '0';
	do_rd_bit <= '1' when (fsm_cur=LATCH) else '0';
	ss_pin <= '1' when (fsm_cur=IDLE or fsm_cur=CONVERSION) else '0';
	sclk_pin <= '1' when (fsm_cur=LATCH) else '0';
	mosi_pin <= thr(9);
	adc_rhr <= rhr;

end guts;
